;------------------------------------------------------------------------
;
;	Baudot receive routine
;
;------------------------------------------------------------------------

		public	InitBaudot
		public	GetBaudotChar
		public	BaudotCnt


;------------ die segmente ----------------------------------------------

?PR?baudot?baudot	segment CODE
?BI?baudot		SEGMENT BIT
?XD?baudot        	SEGMENT XDATA
?DT?baudot		SEGMENT DATA


MAX_BUFFER	equ	256				;256 zeichen grosser buffer (z.B.)
BITTIME		equ	47104
HALFTIME	equ	56320
DATAPIN		equ	P3.2

;----------- globale variablen ----------------------------------------


	RSEG	?BI?baudot
RecStartBit:	dbit	1

	RSEG    ?DT?baudot
BitCnt:		DS	1
DataByte:	ds	1
InCnt:    	ds	2				;counter fuer ringbuffer
OutPtr:		ds	2              			;ptr fuer zeichen ausgabe aus ringbuffer
InPtr:		ds	2				;ptr fuer zeichen eingabe in den ring buffer
	RSEG	?XD?baudot
BufferStart:	ds      MAX_BUFFER			;der Receive buffer selber

BufferEnd	equ	BufferStart + MAX_BUFFER - 1


;------------	die interrupt vectoren ----------------------------------------

		CSEG	at  0003h    			;vector besetzen
		ljmp	INT0Service

		CSEG	at  000bh	
		ljmp	TIMER0Service

;------------- ab hier code beginn ----------------------------------------

RSEG	?PR?baudot?baudot


;------------------------------------------------------------------------
;
;	InitBaudot(void)
;	Baudot eingang initialisieren
;
;------------------------------------------------------------------------

InitBaudot:	
		mov	InPtr,   #high BufferStart  	;in ptr setzen
		mov	InPtr+1, #low  BufferStart
		mov	OutPtr,  #high BufferStart  	;out ptr setzen
		mov	OutPtr+1,#low  BufferStart
		mov	InCnt,#0
		mov	InCnt+1,#0			;counter loeschen
		mov	a,TMOD
		anl	a,#11110000b
		orl	a,#00000001b			;Timer 0 setzen auf 16 Bit Timer
		mov	TMOD,a
		setb	IT0				;negative Flanken triggerung
		clr	TR0
		clr	ET0				; timer interrupt sperren
		setb	EX0				;interrupt freigeben
		ret

;----------------------------------------------------------------------------
;
;	anzahl der zeichen im RX buffer ermitteln
;
;	word	RecBufferCnt(void)
;
;----------------------------------------------------------------------------

BaudotCnt: 	mov	r6,InCnt
		mov	r7,InCnt+1
		ret

;----------------------------------------------------------------------------
;
;	char GetBaudotChar
;
;	zeichen aus ring buffer auslesen
;
;----------------------------------------------------------------------------

GetBaudotChar: 	mov	a,InCnt				;zeichen im buffer ?
		orl	a,InCnt+1
		jz	GetBaudotChar			;wenn nein warten

		mov	dph,OutPtr			;ptr laden
		mov	dpl,OutPtr+1

		movx	a,@dptr				;zeichen aus ring buffer holen
		mov	r7,a				;und in R7 speichern

		inc	dptr				;ptr auf naechste position

;	test ob am buffer ende angekommen

		setb	c
		mov	a,dpl
		subb	a,#low BufferEnd		;ptr > BufferEnd
		mov	a,dph
		subb	a,#high BufferEnd
		jc	GetChar_1			;nein

;	wenn ja ptr neu laden

		mov	OutPtr  ,#high BufferStart
		mov	OutPtr+1,#low  BufferStart
		sjmp	GetChar_2

;	sonst ptr speichern

GetChar_1:	mov	OutPtr  ,dph			;ptr abpspeichern
		mov	OutPtr+1,dpl
GetChar_2:
		mov	a,InCnt+1			;counter - 1
		clr	c
		subb	a,#1
		mov	InCnt+1,a
		mov	a,InCnt
		subb	a,#0
		mov	InCnt,a

		ret

;-------------------------------------------------------------------------
;
;	Service routine fuer Startbit detection
;
;-------------------------------------------------------------------------
INT0Service:	push	acc
		push    psw
		clr	EX0		; Disable more RX interrupts
		setb	RecStartBit	; auf Startbit abfragen
		mov	TL0,#low HALFTIME
		mov	TH0,#high HALFTIME
		setb	TR0		; start timer 0
		setb	ET0		; Timer Interrupt freigeben
		mov	DataByte,#0
		pop	psw
		pop	acc
		reti

;-------------------------------------------------------------------------
;
;	Service routine fuer Timer0 Datenbits einlesen
;
;-------------------------------------------------------------------------

TIMER0Service:	push	acc
		push	psw
		push	dph
		push	dpl

		mov	TL0,#low BITTIME	; Timer neu setzen
		mov	TH0,#high BITTIME

		jb	RecStartBit,RdStrtBit	; Startbit einlesen ?
		mov	a,BitCnt
		xrl	a,#0			; alle 5 Bits eingelesen ?
		jz	RdStopBit
		mov	c,DATAPIN		; bit einlesen und in Databyte einbauen
		mov	a,DataByte
		rrc	a
		mov	DataByte,a
		dec	BitCnt			; counter -1
		sjmp	TM0End			; und fertig

RdStopBit:	mov	c,DataPin		;Stop bit testen
		jnc	FalseStrtBit
		mov	a,DataByte
		rrc	a
		rrc	a
		rrc	a
		anl	a,#00011111b		;Databyte Justieren

		mov	dph,InPtr		;ptr laden und
		mov	dpl,InPtr+1
		movx	@dptr,a			;zeichen speichern
		inc	dptr			;ptr + 1

;	test ob ptr am bufferende angekommen ist

		setb	c
		mov	a,dpl			;ist ptr am buffer ende angekommen ?
		subb	a,#low BufferEnd
		mov	a,dph
		subb	a,#high BufferEnd
		jc	TM0Serv2		;ist nicht am ende

;	wenn ptr am ende angekommen ist ptr neu setzen

		mov	InPtr,  #high BufferStart
		mov	InPtr+1,#low BufferStart
		sjmp	TM0Serv3

;	ptr wieder abspeichern

TM0Serv2:	mov	InPtr,dph
		mov	InPtr+1,dpl

;	buffer voll ?

TM0Serv3:	setb	c
		mov	a,InCnt+1
		subb	a,#low (MAX_BUFFER - 1)
		mov	a,InCnt
		subb    a,#high (MAX_BUFFER - 1)
		jc	TM0Serv4			;buffer ist nicht voll

;	buffer ist voll dann RecOutPtr auf aeltestes element setzen

		mov	OutPtr  ,InPtr
		mov	OutPtr+1,InPtr+1
		sjmp	FalseStrtBit

;	counter incrementieren	wenn buffer noch nicht voll ist

TM0Serv4:	mov	a,InCnt+1
		add	a,#1
		mov	InCnt+1,a
		mov	a,InCnt
		addc	a,#0
		mov	InCnt,a
		sjmp	FalseStrtBit

RdStrtBit:	clr	RecStartBit
		mov	c,DATAPIN
		jc	FalseStrtBit		; falsches Startbit ?
		mov	BitCnt,#5		; 5 Daten bits einlesen
		sjmp	TM0End

FalseStrtBit:	clr	TR0			;Stop Timer
		clr	ET0			;Stop Timer interrupt
		setb	EX0			; wieder auf neuen Start warten

TM0End:		pop	dpl
		pop	dph
		pop	psw
		pop	acc
		reti
		end
